@@ -3,9 +3,10 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 #
-# pyre-strict
 
+import sys
 import textwrap
+import unittest
 from typing import Type
 
 from libcst import parse_module
@@ -32,6 +33,25 @@ class TestApplyAnnotationsVisitor(CodemodTest):
                     return 1
                 """,
             ),
+            (
+                """
+                def foo(
+                    b: str, c: int = ..., *, d: str = ..., e: int, f: int = ...
+                ) -> int: ...
+                """,
+                """
+                def foo(
+                    b, c=5, *, d="a", e, f=10
+                ) -> int:
+                    return 1
+                """,
+                """
+                def foo(
+                    b: str, c: int=5, *, d: str="a", e: int, f: int=10
+                ) -> int:
+                    return 1
+                """,
+            ),
             (
                 """
                 import bar
@@ -608,7 +628,95 @@ class TestApplyAnnotationsVisitor(CodemodTest):
     )
     def test_annotate_functions(self, stub: str, before: str, after: str) -> None:
         context = CodemodContext()
-        ApplyTypeAnnotationsVisitor.add_stub_to_context(
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
             context, parse_module(textwrap.dedent(stub.rstrip()))
         )
         self.assertCodemod(before, after, context_override=context)
+
+    @data_provider(
+        (
+            (
+                """
+                def foo(
+                    a: int, /, b: str, c: int = ..., *, d: str = ..., e: int, f: int = ...
+                ) -> int: ...
+                """,
+                """
+                def foo(
+                    a, /, b, c=5, *, d="a", e, f=10
+                ) -> int:
+                    return 1
+                """,
+                """
+                def foo(
+                    a: int, /, b: str, c: int=5, *, d: str="a", e: int, f: int=10
+                ) -> int:
+                    return 1
+                """,
+            ),
+            (
+                """
+                def foo(
+                    a: int, b: int = ..., /, c: int = ..., *, d: str = ..., e: int, f: int = ...
+                ) -> int: ...
+                """,
+                """
+                def foo(
+                    a, b = 5, /, c = 10, *, d = "a", e, f = 20
+                ) -> int:
+                    return 1
+                """,
+                """
+                def foo(
+                    a: int, b: int = 5, /, c: int = 10, *, d: str = "a", e: int, f: int = 20
+                ) -> int:
+                    return 1
+                """,
+            ),
+        )
+    )
+    @unittest.skipIf(sys.version_info < (3, 8), "Unsupported Python version")
+    def test_annotate_functions_py38(self, stub: str, before: str, after: str) -> None:
+        context = CodemodContext()
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
+            context, parse_module(textwrap.dedent(stub.rstrip()))
+        )
+        self.assertCodemod(before, after, context_override=context)
+
+    @data_provider(
+        (
+            (
+                """
+                def fully_annotated_with_different_stub(a: bool, b: bool) -> str: ...
+                """,
+                """
+                def fully_annotated_with_different_stub(a: int, b: str) -> bool:
+                    return 'hello'
+                """,
+                """
+                def fully_annotated_with_different_stub(a: bool, b: bool) -> str:
+                    return 'hello'
+                """,
+            ),
+        )
+    )
+    def test_annotate_functions_with_existing_annotations(
+        self, stub: str, before: str, after: str
+    ) -> None:
+        context = CodemodContext()
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
+            context, parse_module(textwrap.dedent(stub.rstrip()))
+        )
+        # Test setting the overwrite flag on the codemod instance.
+        self.assertCodemod(
+            before, after, context_override=context, overwrite_existing_annotations=True
+        )
+
+        # Test setting the flag when storing the stub in the context.
+        context = CodemodContext()
+        ApplyTypeAnnotationsVisitor.store_stub_in_context(
+            context,
+            parse_module(textwrap.dedent(stub.rstrip())),
+            overwrite_existing_annotations=True,
+        )
+        self.assertCodemod(before, after, context_override=context)
