@@ -3,7 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-# pyre-strict
 
 import abc
 import builtins
@@ -248,7 +247,7 @@ class _NameUtil:
                         if eval_alias is not None:
                             as_name = eval_alias
                     if full_name.startswith(as_name):
-                        remaining_name = full_name.split(as_name)[1].lstrip(".")
+                        remaining_name = full_name.split(as_name, 1)[1].lstrip(".")
                         results.add(
                             QualifiedName(
                                 f"{real_name}.{remaining_name}"
